package plugins.atournay.jep;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

public class PythonUtils {
    private static PythonUtils instance;

    private PythonUtils() {
    }

    public static PythonUtils getInstance() {
        if (instance == null) {
            instance = new PythonUtils();
        }

        return instance;
    }

    /**
     * List all Conda environments available on the computers
     *
     * @return List of conda environments (Key = Name of environment; Value = Path of the environment)
     */
    public HashMap<String, String> detectCondaEnvironments() {
        HashMap<String, String> condaEnvs = new HashMap<>();
        String[] cmd = new String[]{"conda", "env", "list"};

        try {
            Process process = new ProcessBuilder(cmd).start();

            BufferedReader stdOut = new BufferedReader(new InputStreamReader(process.getInputStream()));

            Pattern pattern = Pattern.compile("#|([^.A-z]conda)|(environments:)|\\*| ");

            List<String> stdOutStr = stdOut.lines().flatMap(pattern::splitAsStream)
                    .collect(Collectors.toList())
                    .stream().filter(e -> !e.equals("")).collect(Collectors.toList());

            for (int i = 0; i < stdOutStr.size(); i += 2) {
                condaEnvs.put(stdOutStr.get(i), stdOutStr.get(i + 1));
            }
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

        return condaEnvs;
    }

    /**
     * Get Conda environment names available on the computer
     *
     * @return List of all Conda environment names
     */
    public Set<String> getCondaEnvNames() {
        return detectCondaEnvironments().keySet();
    }

    /**
     * The Python executable file in Windows is python.exe and in Unix systems /python (which can be linked to a specific version file)
     *
     * @param path The path to a directory where the Python executable must be found
     * @param isDirectory Set if the path is ending by a directory (true) or a file (false)
     * @return The full path of the Python execution file
     */
    public String findPythonExecutable(String path, boolean isDirectory) {
        if (isDirectory) {
            File[] pythonFiles = new File(path).listFiles((file, name) -> name.equals("bin") && file.isDirectory());

            if (pythonFiles != null) {
                for (File binDirectory : pythonFiles) {
                    File[] binFiles = binDirectory.listFiles((file, name) -> name.matches("python?(?:\\.exe|$)"));

                    if (binFiles != null) {
                        return Arrays.stream(binFiles).findFirst().map(File::getAbsolutePath).orElse(null);
                    }
                }
            }

            return null;
        }
        else {
            if (Files.exists(Paths.get(path))) {
                return path;
            }
            else {
                return null;
            }
        }
    }

    /**
     * @param pythonExecutionPath Path to the Python home directory
     * @return The path to the site-packages directory
     */
    public String setSitePackagesDirectory(String pythonExecutionPath) {
        String terminalLine;
        String[] command = new String[]{pythonExecutionPath, "-c", "import sys; print([p for p in sys.path if p.endswith(\"site-packages\")])"};
        String sitePackagesPath = "";

        try {
            // Execute terminal command to find the site-packages directory
            Process process = Runtime.getRuntime().exec(command);

            BufferedReader streamInput = new BufferedReader(new InputStreamReader(process.getInputStream()));
            BufferedReader streamError = new BufferedReader(new InputStreamReader(process.getErrorStream()));

            // Execution result stream output
            while ((terminalLine = streamInput.readLine()) != null) {
                String[] spList = terminalLine.replaceAll("[\\[\\]' ]", "").split(",");

                /*
                    With classical Python interpreter (non-isolated environment like Venv and Conda with its consorts), there can be multiple site-packages folder.
                    So, we must find which one has JEP inside
                 */
                for (String spPath : spList) {
                    String isThereJEP = JepUtils.getInstance().findJepLib(spPath);

                    if (isThereJEP != null) {
                        sitePackagesPath = spPath;
                        break;
                    }
                }
            }

            // Error stream output
            while ((terminalLine = streamError.readLine()) != null) {
                System.err.println(terminalLine);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        return sitePackagesPath;
    }
}
